###################################################
### chunk number 1: 
###################################################
source("~/mestrado/projetos/car-sar/dados/nb.mi.pr")


###################################################
### chunk number 2: 
###################################################
require(spdep)
nbw <- nb2listw(nb.mi.pr)
n.viz <- sapply(nb.mi.pr, length)
w <- nb2mat(nb.mi.pr)
n <- nrow(w)


###################################################
### chunk number 3: 
###################################################
rmvnorm <- function(n, mu, S) 
  rep(mu, each=n) + matrix(rnorm(n*ncol(S)), n)%*%chol(S)



###################################################
### chunk number 4: ricar
###################################################
ricar <- function(nb, sig=1, burn=10) {
  n <- length(nb)
  nviz <- sapply(nb, length)
  sds <- sqrt(sig/nviz)
  env <- new.env()
  assign("x", rnorm(n, sd=sds), envir=env)
  sapply(1:burn, function(i) 
         sapply(1:n, function(j) {
           y <- get("x", envir=env)
           y[j] <- rnorm(1, mean(y[nb[[j]]]), sds[j])
           assign("x", y, envir=env)
         }))
  return(get("x", envir=env))
}
summary(ricar(nb.mi.pr))


###################################################
### chunk number 5: 
###################################################
mcov <- solve(diag(n)-.7*w)%*%diag(1/n.viz)
mcor <- sapply(1:nrow(mcov), function(i)
                mcov[i,]/sqrt(mcov[i,i]*mcov[i,]))

dim(am2 <- rmvnorm(100, rep(0,n), mcov))
round(cov(am2[,1:5]),2)
round(mcov[1:5, 1:5],2)


###################################################
### chunk number 6: 
###################################################
set.seed(1)
m.ams <- t(sapply(1:100, function(k) ricar(nb.mi.pr)))


###################################################
### chunk number 7: 
###################################################
rfit1 <- apply(am2, 1, function(x)
               coef(spautolm(x~1,,nbw,fam='CAR'))[2])
rfit2 <- apply(m.ams, 1, function(x)
               coef(spautolm(x~1,,nbw,fam='CAR'))[2])
rfit3 <- sapply(1:100, function(i)
               coef(spautolm(rnorm(39)~1,,nbw,fam='CAR'))[2])


###################################################
### chunk number 8: 
###################################################
summary(rfit1)
summary(rfit2)
summary(rfit3)


###################################################
### chunk number 9: 
###################################################
post.pcar <- function(pars, y, Q) {
  n <- length(y)
  Q <- Q/exp(pars[n+1])
  dnorm(exp(pars[n+1]), 0, 3, log=TRUE) -
    0.5*(n*log(2*pi)-as.numeric(determinant(Q)$modulus) +
         mahalanobis(pars[1:n], mean(pars[1:n]), Q, TRUE)) +
            sum(dpois(y, exp(pars[1:n]), log=TRUE)) 
}


###################################################
### chunk number 10: 
###################################################
set.seed(3)
b5 <- ricar(nb.mi.pr, 3, 100)
yc <- rpois(39, exp(3+b5))
table(yc)

coef(spautolm(b5~1, listw=nbw, family="CAR"))

Q <- -nb2mat(nb.mi.pr, style="B")
diag(Q) <- -rowSums(Q)

plot(function(x)
     sapply(x,function(a)post.pcar(c(b5,a),yc,Q)), 0,2)
post.pcar(c(b5,0), yc, Q)
post.pcar(c(b5,.5), yc, Q)
post.pcar(c(b5,1), yc, Q)
post.pcar(c(b5,1.5), yc, Q)
post.pcar(c(b5,2), yc, Q)

opt5 <- optim(c(rep(3,39),1), post.pcar, meth='S', hess=T,
              y=yc, Q=Q, control=list(fnscale=-1, maxit=1000))
opt5$conv
opt5$par
exp(opt5$par[40])

sqrt(diag(mc.cov <- solve(-opt5$hess)))

plot(3+b5, opt5$par[1:39])
abline(0,1)

tun <- c(rep(.3,39),20)
v.opt <- diag(tun)%*%mc.cov%*%diag(tun)
sqrt(diag(v.opt))

require(MCMCpack)
system.time(post5 <- MCMCmetrop1R(post.pcar, c(rep(3,39),0), 
                                  1000, 30000, 100, verb=5000,
                                  V=v.opt, ##diag(c(rep(.01,39),.01)),
                                  y=yc, Q=Q))
dim(post5)

par(mfrow=c(6,7), mar=c(2.5,2.5,1,.5), mgp=c(1.5,.5,0))
for (i in 1:ncol(post5))
  plot.ts(post5[,i])

b.fit <- colMeans(post5[,1:39])-mean(post5[,1:39])
summary(b.fit)

cor(b5, b.fit)

sig.am5 <- exp(post5[,40])
summary(sig.am5)

plot(sig.am5)

par(mfrow=c(1,1))
plot(3+b5, colMeans(post5[,1:39]))
abline(0,1)

###acf(post5)


source("/home/elias/mestrado/projetos/car-sar/dados/pl.mi.pr")
colours <- gray(c(.9,.7,.5,.3,.1))
##colours <- c("salmon1", "salmon2", "red3", "brown", "black")

par(mfrow=c(2,2), mar=c(0,0,2,0), mgp=c(1.5,.5,0),
    cex.main=2, font.main=1)
plot(pl.mi.pr, col=colours[findInterval(b5, c(-5,-2,-1,0,1),
                 all.inside=TRUE)], forcefill=FALSE,
     main="Efeito aleatorio simulado")
legend(x=c(-49.5, -48), y=c(-22.5, -24), legend=leglabs(c(-5,-2,-1,0,1)),
       fill=colours, bty="n", cex=2)

plot(pl.mi.pr, col=colours[findInterval(yc, c(0,2,5,10,30),
                 all.inside=TRUE)], forcefill=FALSE,
     main="Variavel Poisson Simulada")
legend(x=c(-49.5, -48), y=c(-22.5, -24), legend=leglabs(c(0,2,5,10,30)),
       fill=colours, bty="n", cex=2)

plot(pl.mi.pr, col=colours[findInterval(b.fit, c(-2,-1,0,1,2),
                 all.inside=TRUE)], forcefill=FALSE,
     main="Efeito aleatorio estimado")
legend(x=c(-49.5, -48), y=c(-22.5, -24), legend=leglabs(c(-2,-1,0,1,2)),
       fill=colours, bty="n", cex=2)




summary(b5)
